﻿Imports Charmed.Sample.Models
Imports Charmed.Sample.Tests.Mocks
Imports Charmed.Sample.ViewModels
Imports Microsoft.VisualStudio.TestPlatform.UnitTestFramework
Imports System.ComponentModel
Imports System.Threading.Tasks
Imports Windows.ApplicationModel.DataTransfer
Imports Windows.UI.Popups
Imports System.Collections.Generic
Imports System

Namespace Charmed.Sample.Tests.ViewModels
    <TestClass>
    Public Class Win8FeedItemViewModelTests
        Private Property ShareManager As ShareManagerMock
        Private Property Serializer As SerializerMock
        Private Property SecondaryPinner As SecondaryPinnerMock
        Private Property Storage As StorageMock

        Private Function GetViewModel() As Win8FeedItemViewModel
            Return New Win8FeedItemViewModel(Me.Serializer, Me.Storage, Me.SecondaryPinner, Me.ShareManager)
        End Function

        Private Property PropertiesChanged As List(Of String)
        Private Sub OnPropertyChanged(ByVal sender As Object, ByVal args As PropertyChangedEventArgs)
            Me.PropertiesChanged.Add(args.PropertyName)
        End Sub

        <TestInitialize>
        Public Sub Init()
            Me.ShareManager = New ShareManagerMock()
            Me.Serializer = New SerializerMock()
            Me.SecondaryPinner = New SecondaryPinnerMock()
            Me.Storage = New StorageMock()

            Me.PropertiesChanged = New List(Of String)
        End Sub

        <TestMethod>
        Public Sub LoadState()
            ' Arrange
            Dim viewModel As Win8FeedItemViewModel = GetViewModel()

            Dim expectedFeedItem As FeedItem = New FeedItem()

            Dim wasInitializeCalled As Boolean = False
            Me.ShareManager.InitializeDelegate = Sub() wasInitializeCalled = True

            ' Act
            viewModel.LoadState(expectedFeedItem, Nothing)

            ' Assert
            Assert.AreSame(expectedFeedItem, viewModel.FeedItem, "FeedItem")
            Assert.IsTrue(wasInitializeCalled, "Share Manager Initialize")
        End Sub

        <TestMethod>
        Public Sub SaveState()
            ' Arrange
            Dim viewModel As Win8FeedItemViewModel = GetViewModel()

            Dim expectedFeedItem As FeedItem = New FeedItem()

            Dim wasCleanupCalled As Boolean = False
            Me.ShareManager.CleanupDelegate = Sub() wasCleanupCalled = True

            ' Act
            viewModel.SaveState(Nothing)

            ' Assert
            Assert.IsTrue(wasCleanupCalled, "Share Manager Initialize")
        End Sub

        <TestMethod>
        Public Sub FeedItem()
            ' Arrange
            Dim viewModel As Win8FeedItemViewModel = GetViewModel()

            Dim expectedFeedItem As FeedItem = New FeedItem()

            ' Act
            AddHandler viewModel.PropertyChanged, AddressOf OnPropertyChanged
            viewModel.FeedItem = expectedFeedItem
            RemoveHandler viewModel.PropertyChanged, AddressOf OnPropertyChanged

            ' Assert
            Assert.IsTrue(Me.PropertiesChanged.Contains("FeedItem"), "Property Changed")
            Assert.AreSame(expectedFeedItem, viewModel.FeedItem, "Feed Item")
        End Sub

        <TestMethod>
        Public Async Function ShareRequested() As Task
            ' Arrange
            Dim viewModel As Win8FeedItemViewModel = GetViewModel()

            Dim dataPackage As DataPackage = New DataPackage()
            Dim dataPackageView As DataPackageView = dataPackage.GetView

            Dim feedItem As FeedItem = New FeedItem With {.Title = Guid.NewGuid().ToString, .Author = Guid.NewGuid().ToString(), .Link = New Uri("http://www.bing.com")}

            viewModel.LoadState(FeedItem, Nothing)

            ' Act
            Me.ShareManager.OnShareRequested()(dataPackage)

            ' Assert
            Assert.IsFalse(String.IsNullOrEmpty(Await dataPackageView.GetTextAsync), "Text Exists")
            Assert.AreEqual(feedItem.Link.ToString, (Await dataPackageView.GetUriAsync()).ToString(), "Uri")
            Assert.IsTrue((Not String.IsNullOrEmpty(Await dataPackageView.GetHtmlFormatAsync)), "HTML Exists")
        End Function

        <TestMethod>
        Public Async Function Pin_PinSucceeded() As Task
            ' Arrange
            Dim viewModel As Win8FeedItemViewModel = GetViewModel()

            Dim feedItem As FeedItem = New FeedItem With {.Title = Guid.NewGuid().ToString, .Author = Guid.NewGuid().ToString(), .Link = New Uri("http://www.bing.com")}

            viewModel.LoadState(FeedItem, Nothing)

            Dim actualPlacement As Placement = Placement.Default
            Dim actualTileInfo As TileInfo = Nothing
            SecondaryPinner.PinDelegate = Function(tileInfo)
                                              actualPlacement = tileInfo.RequestPlacement
                                              actualTileInfo = tileInfo

                                              Return True
                                          End Function

            Dim actualKey As String = ""
            Dim actualPinnedFeedItems As List(Of FeedItem) = Nothing
            Storage.SaveAsyncDelegate = Sub(key, value)
                                            actualKey = key
                                            actualPinnedFeedItems = CType(value, List(Of FeedItem))
                                        End Sub

            ' Act
            Await viewModel.Pin(Nothing)

            ' Assert
            Assert.AreEqual(Placement.Above, actualPlacement, "Placement")
            Assert.AreEqual(String.Format(Constants.SecondaryIdFormat, viewModel.FeedItem.Id), actualTileInfo.TileId, "Tile Info Tile Id")
            Assert.AreEqual(viewModel.FeedItem.Title, actualTileInfo.DisplayName, "Tile Info Display Name")
            Assert.AreEqual(viewModel.FeedItem.Title, actualTileInfo.ShortName, "Tile Info Short Name")
            Assert.AreEqual(viewModel.FeedItem.Id.ToString, actualTileInfo.Arguments, "Tile Info Arguments")
            Assert.AreEqual(Constants.PinnedFeedItemsKey, actualKey, "Save Key")
            Assert.IsNotNull(actualPinnedFeedItems, "Pinned Feed Items")
        End Function

        <TestMethod>
        Public Async Function Pin_PinNotSucceeded() As Task
            ' Arrange
            Dim viewModel As Win8FeedItemViewModel = GetViewModel()

            Dim feedItem As FeedItem = New FeedItem With {.Title = Guid.NewGuid().ToString, .Author = Guid.NewGuid().ToString(), .Link = New Uri("http://www.bing.com")}

            viewModel.LoadState(FeedItem, Nothing)

            Dim actualPlacement As Placement = Placement.Default
            Dim actualTileInfo As TileInfo = Nothing
            SecondaryPinner.PinDelegate = Function(tileInfo)
                                              actualPlacement = tileInfo.RequestPlacement
                                              actualTileInfo = tileInfo

                                              Return False
                                          End Function

            Dim wasSaveCalled As Boolean = False
            Storage.SaveAsyncDelegate = Sub(key, value) wasSaveCalled = True

            ' Act
            Await viewModel.Pin(Nothing)

            ' Assert
            Assert.AreEqual(Placement.Above, actualPlacement, "Placement")
            Assert.AreEqual(String.Format(Constants.SecondaryIdFormat, viewModel.FeedItem.Id), actualTileInfo.TileId, "Tile Info Tile Id")
            Assert.AreEqual(viewModel.FeedItem.Title, actualTileInfo.DisplayName, "Tile Info Display Name")
            Assert.AreEqual(viewModel.FeedItem.Title, actualTileInfo.ShortName, "Tile Info Short Name")
            Assert.AreEqual(viewModel.FeedItem.Id.ToString, actualTileInfo.Arguments, "Tile Info Arguments")
            Assert.IsFalse(wasSaveCalled, "Was Save Called")
        End Function
    End Class
End Namespace
