﻿Imports System
Imports System.Threading
Imports System.Threading.Tasks
Imports System.Collections.Generic

#If NETFX_CORE Then
Imports System.Collections.Concurrent
#End If ' NETFX_CORE

Namespace Charmed.Helpers
    ''' <summary>
    ''' Helper class for calling async methods from synchronous methods.
    ''' </summary>
    ''' <remarks>
    ''' The WinRT version of this implementation is borrowed from Stephen Toub.
    ''' http://blogs.msdn.com/b/pfxteam/archive/2012/01/20/10259049.aspx
    ''' </remarks>
    Public NotInheritable Class AsyncHelper
        ''' <summary>
        ''' Loads data asynchronously, calling the callback when completed.
        ''' </summary>
        ''' <typeparam name="T">The type of the data being loaded</typeparam>
        ''' <param name="callback">The Action that will receive the result</param>
        ''' <param name="loader">The Func that will load the data</param>
        
        Public Shared Sub LoadData(Of T)(ByVal callback As Action(Of T), ByVal loader As Func(Of Task(Of T)))
#If WINDOWS_PHONE Then
			Dim task = loader()
			Dim awaiter = task.GetAwaiter
			awaiter.OnCompleted(Sub()
				If task.Exception IsNot Nothing Then
					Throw task.Exception
				Else
					callback(task.Result)
				End If
			End Sub)
#Else
            Dim prevCtx As SynchronizationContext = SynchronizationContext.Current
            Try
                Dim syncCtx As SingleThreadSynchronizationContext = New SingleThreadSynchronizationContext()
                SynchronizationContext.SetSynchronizationContext(syncCtx)

                Dim tt As Task(Of T) = loader()
                tt.ContinueWith(Sub() syncCtx.Complete(), TaskScheduler.Default)

                syncCtx.RunOnCurrentThread()

                callback(tt.GetAwaiter.GetResult)
            Finally
                SynchronizationContext.SetSynchronizationContext(prevCtx)
            End Try
#End If ' WINDOWS_PHONE
        End Sub

#If NETFX_CORE Then
        ''' <summary>Provides a SynchronizationContext that's single-threaded.</summary>
        Private NotInheritable Class SingleThreadSynchronizationContext
            Inherits SynchronizationContext
            ''' <summary>The queue of work items.</summary>
            Private ReadOnly queue As New BlockingCollection(Of KeyValuePair(Of SendOrPostCallback, Object))()

            ''' <summary>Dispatches an asynchronous message to the synchronization context.</summary>
            ''' <param name="d">The System.Threading.SendOrPostCallback delegate to call.</param>
            ''' <param name="state">The object passed to the delegate.</param>
            Public Overrides Sub Post(ByVal d As SendOrPostCallback, ByVal state As Object)
                If d Is Nothing Then
                    Throw New ArgumentNullException("d")
                End If
                Me.queue.Add(New KeyValuePair(Of SendOrPostCallback, Object)(d, state))
            End Sub

            ''' <summary>Not supported.</summary>
            Public Overrides Sub Send(ByVal d As SendOrPostCallback, ByVal state As Object)
                Throw New NotSupportedException("Synchronously sending is not supported.")
            End Sub

            ''' <summary>Runs an loop to process all queued work items.</summary>
            Public Sub RunOnCurrentThread()
                Dim workItem As KeyValuePair(Of System.Threading.SendOrPostCallback, Object)
                For Each workItem In Me.queue.GetConsumingEnumerable
                    workItem.Key().Invoke(workItem.Value)
                Next workItem
            End Sub

            ''' <summary>Notifies the context that no more work will arrive.</summary>
            Public Sub Complete()
                Me.queue.CompleteAdding()
            End Sub
        End Class
#End If ' NETFX_CORE
    End Class
End Namespace
