﻿Imports System.Reflection
Imports System
Imports System.Collections.Generic

Namespace Charmed.Messaging
    ''' <summary>
    ''' A Message Bus is a central messaging system for
    ''' an application.
    ''' </summary>
    Public NotInheritable Class MessageBus
        Implements IMessageBus
        Private ReadOnly subscribers As New Dictionary(Of Type, List(Of ActionReference))()

        Private ReadOnly lock As New Object()

        ''' <summary>
        ''' Subscribes an action to a particular message type.  When that message type
        ''' is published, this action will be called.
        ''' </summary>
        ''' <typeparam name="TMessage">The type of message to listen for.</typeparam>
        ''' <param name="handler">
        ''' The action which will be called when a message of type <typeparamref name="TMessage"/>
        ''' is published.
        ''' </param>
        Public Sub Subscribe(Of TMessage)(ByVal handler As Action(Of TMessage)) Implements IMessageBus.Subscribe
            SyncLock Me.lock
                If Me.subscribers.ContainsKey(GetType(TMessage)) Then
                    Dim handlers As List(Of ActionReference) = Me.subscribers(GetType(TMessage))
                    handlers.Add(New ActionReference(handler))
                Else
                    Dim handlers As List(Of ActionReference) = New List(Of ActionReference)
                    handlers.Add(New ActionReference(handler))
                    Me.subscribers(GetType(TMessage)) = handlers
                End If
            End SyncLock
        End Sub

        ''' <summary>
        ''' Unsubscribes an action from a particular message type.
        ''' </summary>
        ''' <typeparam name="TMessage">The type of message to stop listening for.</typeparam>
        ''' <param name="handler">
        ''' The action which is to be unsubscribed from messages of type <typeparamref name="TMessage"/>.
        ''' </param>
        Public Sub Unsubscribe(Of TMessage)(ByVal handler As Action(Of TMessage)) Implements IMessageBus.Unsubscribe
            SyncLock Me.lock
                If Me.subscribers.ContainsKey(GetType(TMessage)) Then
                    Dim handlers As List(Of ActionReference) = Me.subscribers(GetType(TMessage))

                    Dim targetReference As ActionReference = Nothing
                    Dim reference As ActionReference
                    For Each reference In handlers
                        Dim action As Action(Of TMessage) = CType(reference.Target, Action(Of TMessage))
                        If action.Target.Equals(handler.Target) AndAlso action.GetMethodInfo.Name.Equals(handler.GetMethodInfo.Name) Then
                            targetReference = reference
                            Exit For
                        End If
                    Next reference
                    handlers.Remove(targetReference)

                    If handlers.Count = 0 Then
                        Me.subscribers.Remove(GetType(TMessage))
                    End If
                End If
            End SyncLock
        End Sub

        ''' <summary>
        ''' Publishes a message to any subscribers of a particular message type.
        ''' </summary>
        ''' <typeparam name="TMessage">The type of message to publish.</typeparam>
        ''' <param name="message">The message to be published</param>
        Public Sub Publish(Of TMessage)(ByVal message As TMessage) Implements IMessageBus.Publish
            Dim subscribers As List(Of Action(Of TMessage)) = Me.RefreshAndGetSubscribers(Of TMessage)()
            For Each subscriber As Action(Of TMessage) In subscribers
                subscriber.Invoke(message)
            Next subscriber
        End Sub

        Private Function RefreshAndGetSubscribers(Of TMessage)() As List(Of Action(Of TMessage))
            Dim toCall As List(Of Action(Of TMessage)) = New List(Of Action(Of TMessage))()
            Dim toRemove As List(Of ActionReference) = New List(Of ActionReference)

            SyncLock Me.lock
                If Me.subscribers.ContainsKey(GetType(TMessage)) Then
                    Dim handlers As List(Of ActionReference) = Me.subscribers(GetType(TMessage))
                    For Each handler As ActionReference In handlers
                        If handler.IsAlive Then
                            toCall.Add(CType(handler.Target, Action(Of TMessage)))
                        Else
                            toRemove.Add(handler)
                        End If
                    Next handler

                    For Each remove As ActionReference In toRemove
                        handlers.Remove(remove)
                    Next remove

                    If handlers.Count = 0 Then
                        subscribers.Remove(GetType(TMessage))
                    End If
                End If
            End SyncLock

            Return toCall
        End Function
    End Class
End Namespace
