﻿Imports System.ComponentModel
Imports System.Runtime.CompilerServices
Imports System.Threading.Tasks
Imports System.Collections.Generic

Namespace Charmed
    ''' <summary>
    ''' A simple base class for view models.
    ''' </summary>
    Public MustInherit Class ViewModelBase
        Implements INotifyPropertyChanged
        Public Event PropertyChanged As PropertyChangedEventHandler Implements INotifyPropertyChanged.PropertyChanged

        ''' <summary>
        ''' Loads state when the view model is loaded.
        ''' </summary>
        ''' <param name="navigationParameter">The navigation parameter.</param>
        ''' <param name="pageState">The state to load.</param>
        Public Overridable Sub LoadState(ByVal navigationParameter As Object, ByVal pageState As Dictionary(Of String, Object))
        End Sub

        ''' <summary>
        ''' Saves state when the view model is unloaded.
        ''' </summary>
        ''' <param name="pageState">The state to save to.</param>
        Public Overridable Sub SaveState(ByVal pageState As Dictionary(Of String, Object))
        End Sub

        ''' <summary>
        ''' Notifies that a property has changed.
        ''' </summary>
        ''' <param name="propertyName">The name of the property that changed.</param>
        Protected Sub NotifyPropertyChanged(ByVal propertyName As String)
            RaiseEvent PropertyChanged(Me, New PropertyChangedEventArgs(propertyName))
        End Sub

        ''' <summary>
        ''' Sets a property to a value and notifies that it changed.
        ''' </summary>
        ''' <typeparam name="T">The type of the property.</typeparam>
        ''' <param name="property">The property reference to set.</param>
        ''' <param name="data">The data to set.</param>
        ''' <param name="propertyName">The name of the property to notify.</param>
        Protected Sub SetProperty(Of T)(ByRef [property] As T, ByVal data As T, Optional ByVal propertyName As String = Nothing)
            If Object.Equals([property], data) Then
                Return
            End If

            [property] = data
            Me.NotifyPropertyChanged(propertyName)
        End Sub

        'INSTANT VB NOTE: The variable isBusy was renamed since Visual Basic does not allow class members with the same name:
        Private isBusy_Renamed As Boolean

        ''' <summary>
        ''' Gets or sets whether the view model is busy.
        ''' </summary>
        Public Property IsBusy As Boolean
            Get
                Return Me.isBusy_Renamed
            End Get
            Set(ByVal value As Boolean)
                Me.SetProperty(Me.isBusy_Renamed, value)
            End Set
        End Property
    End Class

    ''' <summary>
    ''' View model base class that provides strongly typed navigation parameters
    ''' </summary>
    ''' <typeparam name="TParameter"></typeparam>
    Public MustInherit Class ViewModelBase(Of TParameter)
        Inherits ViewModelBase
        Protected ReadOnly serializer As ISerializer

        Public Sub New(ByVal serializer As ISerializer)
            Me.serializer = serializer
        End Sub

        ''' <summary>
        ''' Loads state when the view model is loaded, with a strongly-typed parameter.
        ''' </summary>
        ''' <param name="navigationParameter">The navigation parameter.</param>
        ''' <param name="pageState">The state to load.</param>
        Public MustOverride Overloads Sub LoadState(ByVal navigationParameter As TParameter, ByVal pageState As Dictionary(Of String, Object))

        ''' <summary>
        ''' Loads state when the view model is loaded.
        ''' </summary>
        ''' <param name="navigationParameter">The navigation parameter.</param>
        ''' <param name="pageState">The state to load.</param>
        Public Overrides Sub LoadState(ByVal navigationParameter As Object, ByVal pageState As Dictionary(Of String, Object))
            If navigationParameter IsNot Nothing Then
                Dim serializedParameter As String = navigationParameter.ToString
                If Not String.IsNullOrWhiteSpace(serializedParameter) Then
                    Dim deserializedNavigationParameter As TParameter = Me.serializer.Deserialize(Of TParameter)(navigationParameter.ToString)
                    Me.LoadState(deserializedNavigationParameter, pageState)
                    Return
                End If
            End If

            Me.LoadState(Nothing, pageState)
        End Sub
    End Class
End Namespace
